macroscript MultiDistanceMeter
category:"BodyulCG"
tooltip:"Multi Distance Meter"
(
	local pairs = #()
	local showDistance = false
	local isOpen = false
	
	rollout MultiDistanceMeter "Multi Distance Meter" width:200
	(
		local config = (getDir #plugcfg) + "\MultiDistanceMeter.ini"
		local nodeCb
		
		group "Objects"
		(
			multilistbox mlbxObjects height:8
			button btnAdd "Add" width:84 align:#left across:2
			button btnRemove "Remove" width:84 align:#right
		)
		
		group "Display"
		(
			checkbox chkLines "Lines" checked:true
			checkbox chkObjectNames "Object Names"
			label lblSource "Source:" offset:[0, 4] align:#left	
			radiobuttons rdoSource offset:[36, -18] labels:#("List", "Selection") state:1 offsets:#([-4, 0], [0, 0]) align:#center
			checkbutton cbtnShowDistance "Show Distance" width:180 height:30 align:#center
		)
		
		group "Color Settings"
		(
			label lblLineColor "Line:" align:#left across:2
			colorpicker cpLine height:16 color:(color 150 150 150) align:#right
			label lblTextColor "Text:" align:#left across:2
			colorpicker cpText height:16 color:(color 0 255 130) align:#right
		)

		group "About"
		(
			label lblAbout "Multi Distance Meter v1.1"
			hyperlink hpUrl "bodyulcg.com" \
				address:"https://bodyulcg.com" \
				color:(color 0 255 180) \
				hovercolor:(color 0 255 180) \
				visitedcolor:(color 0 255 180) \
				align:#center
		)
		
		fn drawPair a b =
		(
			local p1 = a.pos
			local p2 = b.pos
			
			if chkLines.checked then
				gw.polyline #(p1, p2) false;
			
			local str = units.formatValue (distance p1 p2)
			
			if chkObjectNames.checked then
				append str (" : " + a.name + " - " + b.name);
			
			gw.text	((p1 + p2) / 2) str color:cpText.color
		)
		
		fn draw =
		(
			gw.setTransform (matrix3 1)
			gw.setColor #line cpLine.color
			
			if rdoSource.state == 1 then
			(
				for i in pairs where isValidNode i[1] and isValidNode i[2] do
					drawPair i[1] i[2];
			)
			else if rdoSource.state == 2 then
			(
				for i = 1 to selection.count - 1 do
					drawPair selection[i] selection[i + 1];
			)

			gw.enlargeUpdateRect #whole
			gw.updateScreen()
		)
		
		fn doesPairExist obj1 obj2 =
		(
			local exist = false
			
			for i in pairs while not exist do
				exist = (i[1] == obj1 or i[1] == obj2) and (i[2] == obj1 or i[2] == obj2);
			
			return exist
		)
		
		fn refreshList =
		(
			mlbxObjects.items = for i in pairs collect
			(
				local name1 = if isValidNode i[1] then i[1].name else "<Delete scene node>"
				local name2 = if isValidNode i[2] then i[2].name else "<Delete scene node>"
				name1 + " - " + name2
			)
		)
		
		fn nodeCallback e n =
			refreshList();
		
		on MultiDistanceMeter open do
		(
			if doesFileExist config then
			(
				setDialogPos MultiDistanceMeter (execute (getIniSetting config "Dialog" "Position"))
				
				chkLines.checked 		= (getIniSetting config "Display" "Lines") as booleanclass
				chkObjectNames.checked 	= (getIniSetting config "Display" "Object Names") as booleanclass
				rdoSource.state 		= (getIniSetting config "Display" "Source") as integer
				
				cpLine.color = execute (getIniSetting config "Color Settings" "Line")
				cpText.color = execute (getIniSetting config "Color Settings" "Text")
			)
			
			nodeCb = nodeeventcallback added:nodeCallback deleted:nodeCallback namechanged:nodeCallback

			refreshList()
			cbtnShowDistance.checked = showDistance
		)
		
		on btnAdd pressed do
		(
			if selection.count < 2 then
			(
				messagebox "Select two or more objects." title:"Multi Distance Meter"
				
				return ok;
			)

			for i = 1 to selection.count - 1 where not doesPairExist selection[i] selection[i + 1] do
				append pairs #(selection[i], selection[i + 1]);

			refreshList()
			redrawViews()
		)
		
		on btnRemove pressed do
		(
			if mlbxObjects.selection.numberset > 0 and mlbxObjects.selection.numberset <= pairs.count then
			(
				local sel = mlbxObjects.selection as array
				
				for i = sel.count to 1 by -1 do
					deleteitem pairs sel[i];
			)
			
			refreshList()
			redrawViews()
		)
		
		on chkLines changed state do
			redrawViews();
		
		on cpLine changed col do
			redrawViews();
		
		on chkObjectNames changed state do
			redrawViews();
		
		on cpText changed col do
			redrawViews();
		
		on rdoSource changed state do
			redrawViews();
		
		on cbtnShowDistance changed state do
		(
			(if state then registerRedrawViewsCallback else unregisterRedrawViewsCallback) draw;
			showDistance = state
			redrawViews()
		)
		
		on MultiDistanceMeter close do
		(
			setIniSetting config "Dialog" "Position" (getDialogPos MultiDistanceMeter as string)
			
			setIniSetting config "Display" "Lines" (chkLines.checked as string)
			setIniSetting config "Display" "Object Names" (chkObjectNames.checked as string)
			setIniSetting config "Display" "Source" (rdoSource.state as string)
			
			setIniSetting config "Color Settings" "Line" (cpLine.color as string)
			setIniSetting config "Color Settings" "Text" (cpText.color as string)
			
			nodeCb = undefined			
			showDistance = cbtnShowDistance.checked
			isOpen = false
			updateToolbarButtons()
			gc light:true
		)
	)
	
	on execute do
	(
		if isOpen then
		(
			destroyDialog MultiDistanceMeter
			isOpen = false
		)
		else
		(
			createDialog MultiDistanceMeter
			isOpen = true
		)		
	)

	on isChecked do
		isOpen;

	on closeDialogs do
		destroyDialog MultiDistanceMeter;
)